@@ -62,6 +62,13 @@ typedef enum {
 
 #define NUMBER_OF_QUEUES 3
 
+
+typedef enum {
+  _NET_WM_BYPASS_COMPOSITOR_HINT_AUTO = 0,
+  _NET_WM_BYPASS_COMPOSITOR_HINT_ON = 1,
+  _NET_WM_BYPASS_COMPOSITOR_HINT_OFF = 2,
+} MetaBypassCompositorHintValue;
+
 struct _MetaWindow
 {
   GObject parent_instance;
@@ -220,6 +227,9 @@ struct _MetaWindow
   
   /* whether net_wm_user_time has been set yet */
   guint net_wm_user_time_set : 1;
+
+  /* whether net_wm_icon_geometry has been set */
+  guint icon_geometry_set : 1;
   
   /* These are the flags from WM_PROTOCOLS */
   guint take_focus : 1;
@@ -391,6 +401,9 @@ struct _MetaWindow
    */
   MetaRectangle user_rect;
   
+  /* Cached net_wm_icon_geometry */
+  MetaRectangle icon_geometry;
+
   /* Requested geometry */
   int border_width;
   /* x/y/w/h here get filled with ConfigureRequest values */
@@ -415,8 +428,7 @@ struct _MetaWindow
   MetaWindow *tile_match;
 
   /* Bypass compositor hints */
-  guint bypass_compositor : 1;
-  guint dont_bypass_compositor : 1;
+  guint bypass_compositor;
 };
 
 struct _MetaWindowClass
@@ -595,9 +607,6 @@ void meta_window_handle_mouse_grab_op_event (MetaWindow *window,
 
 GList* meta_window_get_workspaces (MetaWindow *window);
 
-gboolean meta_window_located_on_workspace (MetaWindow    *window,
-                                           MetaWorkspace *workspace);
-
 void meta_window_get_work_area_current_monitor (MetaWindow    *window,
                                                 MetaRectangle *area);
 void meta_window_get_work_area_for_monitor     (MetaWindow    *window,
