@@ -245,6 +245,36 @@ reload_kwm_win_icon (MetaWindow    *window,
   reload_icon (window, window->display->atom__KWM_WIN_ICON);
 }
 
+static void
+reload_icon_geometry (MetaWindow    *window,
+                      MetaPropValue *value,
+                      gboolean       initial)
+{
+  if (value->type != META_PROP_VALUE_INVALID)
+    {
+      if (value->v.cardinal_list.n_cardinals != 4)
+        {
+          meta_verbose ("_NET_WM_ICON_GEOMETRY on %s has %d values instead of 4\n",
+                        window->desc, value->v.cardinal_list.n_cardinals);
+        }
+      else
+        {
+          MetaRectangle geometry;
+
+          geometry.x = (int)value->v.cardinal_list.cardinals[0];
+          geometry.y = (int)value->v.cardinal_list.cardinals[1];
+          geometry.width = (int)value->v.cardinal_list.cardinals[2];
+          geometry.height = (int)value->v.cardinal_list.cardinals[3];
+
+          meta_window_set_icon_geometry (window, &geometry);
+        }
+    }
+  else
+    {
+      meta_window_set_icon_geometry (window, NULL);
+    }
+}
+
 static void
 reload_struts (MetaWindow    *window,
                MetaPropValue *value,
@@ -1589,53 +1619,25 @@ reload_bypass_compositor (MetaWindow    *window,
                           MetaPropValue *value,
                           gboolean       initial)
 {
-  gboolean requested_value = FALSE;
-  gboolean current_value = window->bypass_compositor;
+  int requested_value = 0;
+  int current_value = window->bypass_compositor;
 
   if (value->type != META_PROP_VALUE_INVALID)
-    {
-      requested_value = ((int) value->v.cardinal == 1);
-      meta_verbose ("Request to bypass compositor for window %s.\n", window->desc);
-    }
+      requested_value = (int) value->v.cardinal;
 
   if (requested_value == current_value)
     return;
 
-  if (requested_value && window->dont_bypass_compositor)
-    {
-      meta_verbose ("Setting bypass and dont compositor for same window (%s) makes no sense, ignoring.\n", window->desc);
-      return;
-    }
+  if (requested_value == _NET_WM_BYPASS_COMPOSITOR_HINT_ON)
+    meta_verbose ("Request to bypass compositor for window %s.\n", window->desc);
+  else if (requested_value == _NET_WM_BYPASS_COMPOSITOR_HINT_OFF)
+    meta_verbose ("Request to don't bypass compositor for window %s.\n", window->desc);
+  else if (requested_value != _NET_WM_BYPASS_COMPOSITOR_HINT_AUTO)
+    return;
 
   window->bypass_compositor = requested_value;
 }
 
-static void
-reload_dont_bypass_compositor (MetaWindow    *window,
-                               MetaPropValue *value,
-                               gboolean       initial)
-{
-  gboolean requested_value = FALSE;
-  gboolean current_value = window->dont_bypass_compositor;
-
-  if (value->type != META_PROP_VALUE_INVALID)
-    {
-      requested_value = ((int) value->v.cardinal == 1);
-      meta_verbose ("Request to don't bypass compositor for window %s.\n", window->desc);
-    }
-
-  if (requested_value == current_value)
-    return;
-
-  if (requested_value && window->bypass_compositor)
-    {
-      meta_verbose ("Setting bypass and dont compositor for same window (%s) makes no sense, ignoring.\n", window->desc);
-      return;
-    }
-
-  window->dont_bypass_compositor = requested_value;
-}
-
 #define RELOAD_STRING(var_name, propname) \
   static void                                       \
   reload_ ## var_name (MetaWindow    *window,       \
@@ -1728,7 +1730,7 @@ meta_display_init_window_prop_hooks (MetaDisplay *display)
     { display->atom_WM_STATE,          META_PROP_VALUE_INVALID,  NULL,                     FALSE, FALSE },
     { display->atom__NET_WM_ICON,      META_PROP_VALUE_INVALID,  reload_net_wm_icon,       FALSE, FALSE },
     { display->atom__KWM_WIN_ICON,     META_PROP_VALUE_INVALID,  reload_kwm_win_icon,      FALSE, FALSE },
-    { display->atom__NET_WM_ICON_GEOMETRY, META_PROP_VALUE_INVALID, NULL,                  FALSE, FALSE },
+    { display->atom__NET_WM_ICON_GEOMETRY, META_PROP_VALUE_CARDINAL_LIST, reload_icon_geometry,     FALSE, FALSE },
     { display->atom_WM_CLIENT_LEADER,  META_PROP_VALUE_INVALID, complain_about_broken_client, FALSE, FALSE },
     { display->atom_SM_CLIENT_ID,      META_PROP_VALUE_INVALID, complain_about_broken_client, FALSE, FALSE },
     { display->atom_WM_WINDOW_ROLE,    META_PROP_VALUE_INVALID, reload_wm_window_role,        FALSE, FALSE },
@@ -1736,7 +1738,6 @@ meta_display_init_window_prop_hooks (MetaDisplay *display)
     { display->atom__NET_WM_STRUT,         META_PROP_VALUE_INVALID, reload_struts,            FALSE, FALSE },
     { display->atom__NET_WM_STRUT_PARTIAL, META_PROP_VALUE_INVALID, reload_struts,            FALSE, FALSE },
     { display->atom__NET_WM_BYPASS_COMPOSITOR, META_PROP_VALUE_CARDINAL,  reload_bypass_compositor, FALSE, FALSE },
-    { display->atom__NET_WM_DONT_BYPASS_COMPOSITOR, META_PROP_VALUE_CARDINAL,  reload_dont_bypass_compositor, FALSE, FALSE },
     { 0 },
   };
 
