@@ -777,7 +777,7 @@ int
 main (int argc, char **argv)
 {
   GtkStyleContext *style;
-  const PangoFontDescription *font_desc;
+  PangoFontDescription *font_desc;
   GtkWidget *window;
   GtkWidget *collection;
   GError *err;
@@ -854,8 +854,9 @@ main (int argc, char **argv)
                     G_CALLBACK (gtk_main_quit), NULL);
 
   gtk_widget_realize (window);
-  style = gtk_widget_get_style_context (window);
-  font_desc = gtk_style_context_get_font (style, 0);
+
+  style = meta_theme_create_style_context (gtk_widget_get_screen (window), NULL);
+  gtk_style_context_get (style, GTK_STATE_FLAG_NORMAL, "font", &font_desc, NULL);
 
   g_assert (style);
   g_assert (font_desc);
@@ -890,7 +891,9 @@ main (int argc, char **argv)
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
                             collection,
                             gtk_label_new (_("Benchmark")));
-  
+
+  pango_font_description_free (font_desc);
+
   i = 0;
   while (i < (int) G_N_ELEMENTS (previews))
     {
@@ -925,15 +928,17 @@ get_flags (GtkWidget *widget)
 }
 
 static int
-get_text_height (GtkWidget *widget)
+get_text_height (GtkWidget       *widget,
+                 GtkStyleContext *style)
 {
-  GtkStyleContext *style;
-  const PangoFontDescription *font_desc;
+  PangoFontDescription *font_desc;
+  int text_height;
 
-  style = gtk_widget_get_style_context (widget);
-  font_desc = gtk_style_context_get_font (style, 0);
-  return meta_pango_font_desc_get_text_height (font_desc,
-                                               gtk_widget_get_pango_context (widget));
+  gtk_style_context_get (style, GTK_STATE_FLAG_NORMAL, "font", &font_desc, NULL);
+  text_height = meta_pango_font_desc_get_text_height (font_desc,
+                                                      gtk_widget_get_pango_context (widget));
+  pango_font_description_free (font_desc);
+  return text_height;
 }
 
 static PangoLayout*
@@ -950,6 +955,7 @@ static void
 run_theme_benchmark (void)
 {
   GtkWidget* widget;
+  GtkStyleContext *style_context;
   cairo_surface_t *pixmap;
   MetaFrameBorders borders;
   MetaButtonState button_states[META_BUTTON_TYPE_LAST] =
@@ -973,13 +979,15 @@ run_theme_benchmark (void)
   
   widget = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_widget_realize (widget);
-  
+
+  style_context = meta_theme_create_style_context (gtk_widget_get_screen (widget), NULL);
+
   meta_theme_get_frame_borders (global_theme,
                                 META_FRAME_TYPE_NORMAL,
-                                get_text_height (widget),
+                                get_text_height (widget, style_context),
                                 get_flags (widget),
                                 &borders);
-  
+
   layout = create_title_layout (widget);
   
   i = 0;
@@ -1019,13 +1027,13 @@ run_theme_benchmark (void)
       cr = cairo_create (pixmap);
 
       meta_theme_draw_frame (global_theme,
-                             widget,
+                             style_context,
                              cr,
                              META_FRAME_TYPE_NORMAL,
                              get_flags (widget),
                              client_width, client_height,
                              layout,
-                             get_text_height (widget),
+                             get_text_height (widget, style_context),
                              &button_layout,
                              button_states,
                              meta_preview_get_mini_icon (),
@@ -1053,6 +1061,7 @@ run_theme_benchmark (void)
 
   g_timer_destroy (timer);
   g_object_unref (G_OBJECT (layout));
+  g_object_unref (style_context);
   gtk_widget_destroy (widget);
 
 #undef ITERATIONS
